/* WWWglance.c - generate an HTML gopher tree file from walker output */
/* Foteos Macrides -- WFEB */
/* Modification of:        */
/* ================        */
/* glance.c - generate a "gopher tree at a glance" file from walker output */
/* Bruce Tanner -- Cerritos College */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

struct {
    char type;
    char *value;
} graphics[] = {{'0'," (FILE)"},{'1'," (DIR)"}, {'2'," (CSO)"},
		{'4'," (HQX)"}, {'5'," (BIN)"}, {'7'," (?)"},
		{'8'," (TEL)"}, {'9'," (BIN)"}, {'I'," (IMG)"},
		{'T'," (3270)"},{'s'," (SND)"}, {'g'," (IMG)"},
		{'M'," (MIME)"},{'h'," (HTML)"},{';'," (MOV)"},
		{'\0', ""}};

char *get_graphic(char);
char *URLescape(char *);

static char *hex = "0123456789ABCDEF";	/* For escaping URL's */
static unsigned char isAcceptable[96] =	/* For escaping URL's */
    /*   0 1 2 3 4 5 6 7 8 9 A B C D E F */
    {    0,0,0,0,0,0,0,0,0,0,7,6,0,7,7,7,	/* 2x   !"#$%&'()*+,-./	 */
         7,7,7,7,7,7,7,7,7,7,0,0,0,0,0,0,	/* 3x  0123456789:;<=>?	 */
	 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* 4x  @ABCDEFGHIJKLMNO  */
	 7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,7,	/* 5X  PQRSTUVWXYZ[\]^_	 */
	 0,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* 6x  `abcdefghijklmno	 */
	 7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,0 };	/* 7X  pqrstuvwxyz{\}~	DEL */

main(int argc, char *argv[])
{
    char *cp, in_line[500], prefix[200], selector[500], title[500];
    char URL[500], Path[500], *path;
    int level;
    unsigned short Info_Type;
    FILE *in, *out;

    if (argc != 3) {
        printf("usage: WWWglance input output\n");
        exit(1);
    }

    if ((in = fopen(argv[1], "r")) == NULL) {
        perror("open input");
        exit(1);
    }
    
    if ((out = fopen(argv[2], "w")) == NULL) {
        perror("open output");
        exit(1);
    }

    fprintf(out, "<HTML>\n<HEAD>\n");
    fprintf(out, "<TITLE>Gopher Directory Tree</TITLE>\n</HEAD>\n\n");
    fprintf(out, "<BODY>\n<H1>Gopher Directory Tree</H1>\n<PRE>");
    while (fgets(in_line, sizeof(in_line), in)) {
        if (cp = strchr(in_line, '\n'))
            *cp = '\0';                                 /* remove newline */
        if (strncmp(in_line, "Title: ", 7) == 0) {      /* Title line? */
            strcpy(title, in_line + 10);                /* yes, save it */
            level = atoi(in_line + 7);                  /* get level */
        }
        if (strncmp(in_line, "Selector: ", 10) == 0) {  /* Selector line? */
            prefix[0] = '\0';
            while (level) {                             /* prefix the level */
                strcat(prefix, "| ");
                level--;
            }
            strcpy(selector, in_line + 10);             /* make clean copy */
	    Info_Type = FALSE;
	    if (selector[0] == 'h' &&		        /* HTML?      */
	        !strncmp(selector+1, "GET", 3) &&       /* on an http */
	        (cp=strchr(selector+1, '/'))) {         /*  server?   */
		strcpy(URL, "http://");		        /* yes        */
	        strcpy(Path, cp+1);
	    } else if (selector[0] == '8') {	        /* telnet? */
		strcpy(URL, "telnet://");	        /* yes     */
		if ((cp=strchr(selector+1, '|')) != NULL
		    && cp != selector+1) {
		    *cp = '\0';
		    strcat(URL, selector+1);
		    strcat(URL, "@");
		    *cp = '|';
		    strcpy(Path, cp);
		} else {
		    strcpy(Path, selector+1);
		}
	    } else if (selector[0] == 'T') {	        /* tn3270? */
		strcpy(URL, "tn3270://");	        /* yes     */
		if ((cp=strchr(selector+1, '|')) != NULL
		    && cp != selector+1) {
		    *cp = '\0';
		    strcat(URL, selector+1);
		    strcat(URL, "@");
		    *cp = '|';
		    strcpy(Path, cp);
		} else {
		    strcpy(Path, selector+1);
		}
	    } else if (selector[0] == 'i') {	        /* Type=i? */
	        Info_Type = TRUE;		        /* yes     */
	    } else {
	        strcpy(URL, "gopher://");	        /* gopher URL */
		strcpy(Path, selector);		        /* by default */
	    }
	    if (!Info_Type && (cp=strchr(Path,'|'))) {  /* host? */
	        *cp = '\0';			        /* yes   */
		strcat(URL, cp+1);
		if (cp=strchr(URL, '|'))	        /* port? */
		    *cp = ':';			        /* yes   */
		strcat(URL, "/");
	    } else {				        /* default to a */
	        URL[0] = '\0';			        /*  partial URL */
	    }
	    if (!Info_Type) {			        /* Type=i?       */
	        path = URLescape(Path);		        /* no, so escape */
	        strcat(URL, path);                      /*  and use the  */
		free(path);			        /*  path for a   */
                fprintf(out,			        /*  URL          */
			"%s<A HREF=\"%s\"\n  TITLE=\"%s\"\n  >%s</A>%s\n",
			prefix, URL,
			title,
			title, get_graphic(selector[0]));
	    } else {					/* is Type=i */
	        fprintf(out, "%s%s\n", prefix, title);  /* use text  */
	    }
        }
    }
    fprintf(out, "</PRE>\n</BODY>\n</HTML>\n");
    fclose(in);
    fclose(out);
}


char *get_graphic(char ch)
{
    int ind;

    for (ind = 0; graphics[ind].type; ind++)
        if (graphics[ind].type == ch)
            return graphics[ind].value;
    return "";
}


char * URLescape(char * str)
{
#define ACCEPTABLE(a)	( a>=32 && a<128 && ((isAcceptable[a-32]) & 1))
    char *p;
    char *q;
    char *result;
    int unacceptable = 0;
    for(p=str; *p; p++)
        if (!ACCEPTABLE((unsigned char)*p))
		unacceptable++;
    result = (char *) malloc(p-str + unacceptable+ unacceptable + 1);
    for(q=result, p=str; *p; p++) {
    	unsigned char a = *p;
	if (!ACCEPTABLE(a)) {
	    *q++ = '%';	/* Means hex commming */
	    *q++ = hex[a >> 4];
	    *q++ = hex[a & 15];
	}
	else *q++ = *p;
    }
    *q++ = '\0';			/* Terminate */
    return(result);
}
